@@ -2,18 +2,21 @@
 from __future__ import division, print_function
 
 from functools import partial
-
-import pytest
 import warnings
+
 import numpy as np
+import pytest
+
+from pandas.compat.numpy import _np_version_under1p13
+import pandas.util._test_decorators as td
+
+from pandas.core.dtypes.common import is_integer_dtype
 
 import pandas as pd
 from pandas import Series, isna
-from pandas.core.dtypes.common import is_integer_dtype
+from pandas.core.arrays import DatetimeArray
 import pandas.core.nanops as nanops
 import pandas.util.testing as tm
-import pandas.util._test_decorators as td
-from pandas.compat.numpy import _np_version_under1p13
 
 use_bn = nanops._USE_BOTTLENECK
 
@@ -24,21 +27,21 @@ class TestnanopsDataFrame(object):
         np.random.seed(11235)
         nanops._USE_BOTTLENECK = False
 
-        self.arr_shape = (11, 7, 5)
+        arr_shape = (11, 7, 5)
 
-        self.arr_float = np.random.randn(*self.arr_shape)
-        self.arr_float1 = np.random.randn(*self.arr_shape)
+        self.arr_float = np.random.randn(*arr_shape)
+        self.arr_float1 = np.random.randn(*arr_shape)
         self.arr_complex = self.arr_float + self.arr_float1 * 1j
-        self.arr_int = np.random.randint(-10, 10, self.arr_shape)
-        self.arr_bool = np.random.randint(0, 2, self.arr_shape) == 0
+        self.arr_int = np.random.randint(-10, 10, arr_shape)
+        self.arr_bool = np.random.randint(0, 2, arr_shape) == 0
         self.arr_str = np.abs(self.arr_float).astype('S')
         self.arr_utf = np.abs(self.arr_float).astype('U')
         self.arr_date = np.random.randint(0, 20000,
-                                          self.arr_shape).astype('M8[ns]')
+                                          arr_shape).astype('M8[ns]')
         self.arr_tdelta = np.random.randint(0, 20000,
-                                            self.arr_shape).astype('m8[ns]')
+                                            arr_shape).astype('m8[ns]')
 
-        self.arr_nan = np.tile(np.nan, self.arr_shape)
+        self.arr_nan = np.tile(np.nan, arr_shape)
         self.arr_float_nan = np.vstack([self.arr_float, self.arr_nan])
         self.arr_float1_nan = np.vstack([self.arr_float1, self.arr_nan])
         self.arr_nan_float1 = np.vstack([self.arr_nan, self.arr_float1])
@@ -46,22 +49,22 @@ class TestnanopsDataFrame(object):
 
         self.arr_inf = self.arr_float * np.inf
         self.arr_float_inf = np.vstack([self.arr_float, self.arr_inf])
-        self.arr_float1_inf = np.vstack([self.arr_float1, self.arr_inf])
-        self.arr_inf_float1 = np.vstack([self.arr_inf, self.arr_float1])
-        self.arr_inf_inf = np.vstack([self.arr_inf, self.arr_inf])
 
         self.arr_nan_inf = np.vstack([self.arr_nan, self.arr_inf])
         self.arr_float_nan_inf = np.vstack([self.arr_float, self.arr_nan,
                                             self.arr_inf])
-        self.arr_nan_float1_inf = np.vstack([self.arr_float, self.arr_inf,
-                                             self.arr_nan])
         self.arr_nan_nan_inf = np.vstack([self.arr_nan, self.arr_nan,
                                           self.arr_inf])
-        self.arr_obj = np.vstack([self.arr_float.astype(
-            'O'), self.arr_int.astype('O'), self.arr_bool.astype(
-                'O'), self.arr_complex.astype('O'), self.arr_str.astype(
-                    'O'), self.arr_utf.astype('O'), self.arr_date.astype('O'),
-            self.arr_tdelta.astype('O')])
+        self.arr_obj = np.vstack([
+            self.arr_float.astype('O'),
+            self.arr_int.astype('O'),
+            self.arr_bool.astype('O'),
+            self.arr_complex.astype('O'),
+            self.arr_str.astype('O'),
+            self.arr_utf.astype('O'),
+            self.arr_date.astype('O'),
+            self.arr_tdelta.astype('O')
+        ])
 
         with np.errstate(invalid='ignore'):
             self.arr_nan_nanj = self.arr_nan + self.arr_nan * 1j
@@ -74,51 +77,19 @@ class TestnanopsDataFrame(object):
 
         self.arr_float_2d = self.arr_float[:, :, 0]
         self.arr_float1_2d = self.arr_float1[:, :, 0]
-        self.arr_complex_2d = self.arr_complex[:, :, 0]
-        self.arr_int_2d = self.arr_int[:, :, 0]
-        self.arr_bool_2d = self.arr_bool[:, :, 0]
-        self.arr_str_2d = self.arr_str[:, :, 0]
-        self.arr_utf_2d = self.arr_utf[:, :, 0]
-        self.arr_date_2d = self.arr_date[:, :, 0]
-        self.arr_tdelta_2d = self.arr_tdelta[:, :, 0]
 
         self.arr_nan_2d = self.arr_nan[:, :, 0]
         self.arr_float_nan_2d = self.arr_float_nan[:, :, 0]
         self.arr_float1_nan_2d = self.arr_float1_nan[:, :, 0]
         self.arr_nan_float1_2d = self.arr_nan_float1[:, :, 0]
-        self.arr_nan_nan_2d = self.arr_nan_nan[:, :, 0]
-        self.arr_nan_nanj_2d = self.arr_nan_nanj[:, :, 0]
-        self.arr_complex_nan_2d = self.arr_complex_nan[:, :, 0]
-
-        self.arr_inf_2d = self.arr_inf[:, :, 0]
-        self.arr_float_inf_2d = self.arr_float_inf[:, :, 0]
-        self.arr_nan_inf_2d = self.arr_nan_inf[:, :, 0]
-        self.arr_float_nan_inf_2d = self.arr_float_nan_inf[:, :, 0]
-        self.arr_nan_nan_inf_2d = self.arr_nan_nan_inf[:, :, 0]
 
         self.arr_float_1d = self.arr_float[:, 0, 0]
         self.arr_float1_1d = self.arr_float1[:, 0, 0]
-        self.arr_complex_1d = self.arr_complex[:, 0, 0]
-        self.arr_int_1d = self.arr_int[:, 0, 0]
-        self.arr_bool_1d = self.arr_bool[:, 0, 0]
-        self.arr_str_1d = self.arr_str[:, 0, 0]
-        self.arr_utf_1d = self.arr_utf[:, 0, 0]
-        self.arr_date_1d = self.arr_date[:, 0, 0]
-        self.arr_tdelta_1d = self.arr_tdelta[:, 0, 0]
 
         self.arr_nan_1d = self.arr_nan[:, 0, 0]
         self.arr_float_nan_1d = self.arr_float_nan[:, 0, 0]
         self.arr_float1_nan_1d = self.arr_float1_nan[:, 0, 0]
         self.arr_nan_float1_1d = self.arr_nan_float1[:, 0, 0]
-        self.arr_nan_nan_1d = self.arr_nan_nan[:, 0, 0]
-        self.arr_nan_nanj_1d = self.arr_nan_nanj[:, 0, 0]
-        self.arr_complex_nan_1d = self.arr_complex_nan[:, 0, 0]
-
-        self.arr_inf_1d = self.arr_inf.ravel()
-        self.arr_float_inf_1d = self.arr_float_inf[:, 0, 0]
-        self.arr_nan_inf_1d = self.arr_nan_inf[:, 0, 0]
-        self.arr_float_nan_inf_1d = self.arr_float_nan_inf[:, 0, 0]
-        self.arr_nan_nan_inf_1d = self.arr_nan_nan_inf[:, 0, 0]
 
     def teardown_method(self, method):
         nanops._USE_BOTTLENECK = use_bn
@@ -141,12 +112,12 @@ class TestnanopsDataFrame(object):
             if axis != 0 and hasattr(
                     targ, 'shape') and targ.ndim and targ.shape != res.shape:
                 res = np.split(res, [targ.shape[0]], axis=0)[0]
-        except:
+        except (ValueError, IndexError):
             targ, res = _coerce_tds(targ, res)
 
         try:
             tm.assert_almost_equal(targ, res, check_dtype=check_dtype)
-        except:
+        except AssertionError:
 
             # handle timedelta dtypes
             if hasattr(targ, 'dtype') and targ.dtype == 'm8[ns]':
@@ -167,11 +138,11 @@ class TestnanopsDataFrame(object):
                 else:
                     try:
                         res = res.astype('c16')
-                    except:
+                    except RuntimeError:
                         res = res.astype('f8')
                     try:
                         targ = targ.astype('c16')
-                    except:
+                    except RuntimeError:
                         targ = targ.astype('f8')
             # there should never be a case where numpy returns an object
             # but nanops doesn't, so make that an exception
@@ -359,6 +330,7 @@ class TestnanopsDataFrame(object):
 
     def test_nanmedian(self):
         with warnings.catch_warnings(record=True):
+            warnings.simplefilter("ignore", RuntimeWarning)
             self.check_funs(nanops.nanmedian, np.median, allow_complex=False,
                             allow_str=False, allow_date=False,
                             allow_tdelta=True, allow_obj='convert')
@@ -394,12 +366,14 @@ class TestnanopsDataFrame(object):
 
     def test_nanmin(self):
         with warnings.catch_warnings(record=True):
+            warnings.simplefilter("ignore", RuntimeWarning)
             func = partial(self._minmax_wrap, func=np.min)
             self.check_funs(nanops.nanmin, func,
                             allow_str=False, allow_obj=False)
 
     def test_nanmax(self):
-        with warnings.catch_warnings(record=True):
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", RuntimeWarning)
             func = partial(self._minmax_wrap, func=np.max)
             self.check_funs(nanops.nanmax, func,
                             allow_str=False, allow_obj=False)
@@ -417,6 +391,7 @@ class TestnanopsDataFrame(object):
 
     def test_nanargmax(self):
         with warnings.catch_warnings(record=True):
+            warnings.simplefilter("ignore", RuntimeWarning)
             func = partial(self._argminmax_wrap, func=np.argmax)
             self.check_funs(nanops.nanargmax, func,
                             allow_str=False, allow_obj=False,
@@ -424,6 +399,7 @@ class TestnanopsDataFrame(object):
 
     def test_nanargmin(self):
         with warnings.catch_warnings(record=True):
+            warnings.simplefilter("ignore", RuntimeWarning)
             func = partial(self._argminmax_wrap, func=np.argmin)
             self.check_funs(nanops.nanargmin, func, allow_str=False,
                             allow_obj=False)
@@ -459,7 +435,6 @@ class TestnanopsDataFrame(object):
                             allow_str=False, allow_date=False,
                             allow_tdelta=False)
 
-    @td.skip_if_no("numpy", min_version="1.10.0")
     def test_nanprod(self):
         self.check_funs(nanops.nanprod, np.prod, allow_str=False,
                         allow_date=False, allow_tdelta=False,
@@ -994,6 +969,26 @@ class TestNankurtFixedValues(object):
         return np.random.RandomState(1234)
 
 
+class TestDatetime64NaNOps(object):
+    @pytest.mark.parametrize('tz', [None, 'UTC'])
+    @pytest.mark.xfail(reason="disabled")
+    # Enabling mean changes the behavior of DataFrame.mean
+    # See https://github.com/pandas-dev/pandas/issues/24752
+    def test_nanmean(self, tz):
+        dti = pd.date_range('2016-01-01', periods=3, tz=tz)
+        expected = dti[1]
+
+        for obj in [dti, DatetimeArray(dti), Series(dti)]:
+            result = nanops.nanmean(obj)
+            assert result == expected
+
+        dti2 = dti.insert(1, pd.NaT)
+
+        for obj in [dti2, DatetimeArray(dti2), Series(dti2)]:
+            result = nanops.nanmean(obj)
+            assert result == expected
+
+
 def test_use_bottleneck():
 
     if nanops._BOTTLENECK_INSTALLED:
@@ -1036,3 +1031,29 @@ def test_numpy_ops_np_version_under1p13(numpy_op, expected):
             assert result == expected
     else:
         assert result == expected
+
+
+@pytest.mark.parametrize("operation", [
+    nanops.nanany,
+    nanops.nanall,
+    nanops.nansum,
+    nanops.nanmean,
+    nanops.nanmedian,
+    nanops.nanstd,
+    nanops.nanvar,
+    nanops.nansem,
+    nanops.nanargmax,
+    nanops.nanargmin,
+    nanops.nanmax,
+    nanops.nanmin,
+    nanops.nanskew,
+    nanops.nankurt,
+    nanops.nanprod,
+])
+def test_nanops_independent_of_mask_param(operation):
+    # GH22764
+    s = pd.Series([1, 2, np.nan, 3, np.nan, 4])
+    mask = s.isna()
+    median_expected = operation(s)
+    median_result = operation(s, mask=mask)
+    assert median_expected == median_result
